<?php

declare(strict_types=1);

namespace Dedi\SyliusMailAdminPlugin\DependencyInjection\Compiller;

use Dedi\SyliusMailAdminPlugin\Utils\ArrayMerge;
use Symfony\Component\DependencyInjection\Compiler\CompilerPassInterface;
use Symfony\Component\DependencyInjection\ContainerBuilder;
use Symfony\Component\DependencyInjection\Definition;
use Symfony\Component\Yaml\Yaml;

class MailConfigurationPass implements CompilerPassInterface
{
    public function process(ContainerBuilder $container): void
    {
        if (!$container->has('dedi_sylius_mail_admin_plugin.form_registry.mail_variable_definition')) {
            return;
        }

        $mailConfigurations = $container->getExtensionConfig('dedi_sylius_mail_admin');
        $mailVariableRegistry = $container->getDefinition('dedi_sylius_mail_admin_plugin.form_registry.mail_variable_definition');

        $defaultMailConfiguration = [
            [
                'mails' => Yaml::parseFile(__DIR__ . '/../../Resources/config/default_mail_variables.yaml'),
            ],
        ];
        $mailConfigurations = ArrayMerge::arrayMergeRecursiveDistinct($defaultMailConfiguration, $mailConfigurations);

        if (!empty($mailConfigurations) && isset($mailConfigurations[0]['mails'])) {
            foreach ($mailConfigurations[0]['mails'] as $code => $mailConfiguration) {
                if (!empty($mailConfiguration['variables'])) {
                    $this->processMailVariablesConfiguration(
                        $mailConfiguration['variables'],
                        $mailVariableRegistry,
                        $code
                    );
                }
            }
        }
    }

    private function processMailVariablesConfiguration(array $variablesConfiguration, Definition $mailVariableRegistry, string $code): void
    {
        foreach ($variablesConfiguration as $name => $variableDefinition) {
            $mailVariableRegistry->addMethodCall(
                'addVariableDefinition',
                [$code, $name, $variableDefinition['label']]
            );
        }
    }
}
